@@ -1105,12 +1105,10 @@ class MBIODevice(object):
         pass
 
     def auto(self):
-        for value in self.values:
-            value.auto()
+        self.values.auto()
 
-    # def manual(self):
-        # for value in self.values:
-            # value.manual()
+    def isManualValue(self):
+        return self.values.isManual()
 
     def __getitem__(self, key):
         return self.values[key]
