@@ -20,6 +20,7 @@ from .mbiosocket import MBIOSocketString
 from .device import MBIODevice
 from prettytable import PrettyTable
 import pprint
+import pathlib
 
 from .xmlconfig import XMLConfig
 
@@ -46,6 +47,7 @@ class MBIODeviceDigimatSIO(MBIODevice):
 
         self.config.set('watchdog', 60)
         self.config.set('boards', 1)
+        self.config.set('extended', False)
 
         self.DI=[]
         self.DO=[]
@@ -88,6 +90,9 @@ class MBIODeviceDigimatSIO(MBIODevice):
     def onLoad(self, xml: XMLConfig):
         self.config.update('watchdog', xml.getInt('watchdog'))
         self.config.update('boards', xml.getInt('boards', vmin=1, vmax=7))
+        self.config.update('extended', xml.getBool('extended'))
+        if self.config.extended:
+            self.NBCHANNELDI=8
 
         for board in range(self.config.boards):
             self.LEDR.append(self.valueDigital('ledr%d' % board, writable=True))
@@ -581,6 +586,7 @@ class SIOConfigurator(object):
         self._logger=logger
         self._config={}
         self._updated=False
+        self._response=None
         self.reset()
 
     @property
@@ -590,6 +596,7 @@ class SIOConfigurator(object):
     def reset(self):
         self._config={}
         self._updated=False
+        self._response=None
 
     @property
     def MAC(self):
@@ -662,7 +669,7 @@ class SIOConfigurator(object):
             url += '/%s' % path
         return url
 
-    def post(self, path, data=None):
+    def post(self, path, data=None, timeout=3):
         try:
             url=self.url(path)
             headers={'api_token': self._token}
@@ -670,11 +677,10 @@ class SIOConfigurator(object):
             if data is None:
                 data={}
 
-            r=requests.post(url, headers=headers, json=data, verify=False, timeout=2.0, proxies=proxies)
+            r=requests.post(url, headers=headers, json=data, verify=False, timeout=timeout, proxies=proxies)
             if r and r.ok:
-                data=r.json()
-                if data and data['status']=='success':
-                    return True
+                self._response=r.text
+                return True
         except:
             pass
 
@@ -686,11 +692,12 @@ class SIOConfigurator(object):
             if data is None:
                 data={}
 
-            r=requests.get(url, headers=headers, params=data, verify=False, timeout=2.0, proxies=proxies)
+            r=requests.get(url, headers=headers, params=data, verify=False, timeout=3.0, proxies=proxies)
             if r and r.ok:
                 data=r.json()
                 return data
         except:
+            # self.logger.exception('get')
             pass
 
     def connect(self):
@@ -719,7 +726,7 @@ class SIOConfigurator(object):
     def reboot(self):
         if self.connect():
             api='system/reboot'
-            if self.post(api):
+            if self.post(api, timeout=10):
                 return True
         return False
 
@@ -756,7 +763,7 @@ class SIOConfigurator(object):
     def upgrade(self, url):
         if url and self.connect():
             api='system/update'
-            if self.post(api, {'url': url}):
+            if self.post(api, {'url': url}, timeout=60):
                 return True
         return False
 
@@ -801,27 +808,28 @@ class SIOScanner(MBIONetworkScanner):
         return None
 
     def retrieveFirmware(self, download=False) -> Version:
-        if not self._firmware:
-            fname='sio.bin'
-            falias='sio-firmware.bin'
+        fname='sio.bin'
+        falias='sio-firmware.bin'
+        fpath='/tmp/%s' % fname
+        p=pathlib.Path(fpath)
 
-            if download:
-                self.downloadFirmware('/tmp/%s' % fname)
+        if download or not p.is_file():
+            self.downloadFirmware('/tmp/%s' % fname)
 
-            url=self.mbio.wsExposeFile('/tmp/%s' % fname, alias=falias)
-            if not url:
-                url=self.mbio.wsExposeFile('/etc/sysconfig/digimat/firmware/%s' % fname, alias=falias)
+        url=self.mbio.wsExposeFile('/tmp/%s' % fname, alias=falias)
+        if not url:
+            url=self.mbio.wsExposeFile('/etc/sysconfig/digimat/firmware/%s' % fname, alias=falias)
 
-            data=self.mbio.wsGetFileContent(falias)
-            version=self.extractFirmwareVersion(data)
+        data=self.mbio.wsGetFileContent(falias)
+        version=self.extractFirmwareVersion(data)
 
-            if version:
-                self._firmware={'name': falias, 'url': url,
-                                'version': version['software']}
-                self.logger.info('Found SIO firmware %s' % self._firmware)
+        if version:
+            self._firmware={'name': falias, 'url': url,
+                            'version': version['software']}
+            self.logger.info('Found SIO firmware %s' % self._firmware)
 
-            # this may contain a "huge" amount of RAM (>800kB)
-            del data
+        # this may contain a "huge" amount of RAM (>800kB)
+        del data
 
         return self._firmware
 
@@ -836,6 +844,9 @@ class SIOScanner(MBIONetworkScanner):
                         (host, c.version, c.hwversion))
                     if c.upgrade(self._firmware['url']):
                         result=True
+
+                    # FIXME: to be removed
+                    self.logger.warning('UPGRADE POST response is [%s]' % c._response)
                 else:
                     self.logger.warning('No valid firmware found for SIO device %s [%s:%s]' %
                         (host, c.version, c.hwversion))
@@ -843,7 +854,36 @@ class SIOScanner(MBIONetworkScanner):
                 return result
         return False
 
-    def upgradeAll(self, fname=None):
+    def retrieveOnlineDevices(self):
+        devices={}
+        for g in self.getMBIO().gateways.all():
+            if g.isOpen():
+                d=g.device(1)
+                if d and d.isOnline() and d.model=='SIO':
+                    host=g.host
+                    c=self.configurator(host)
+                    if c.connect():
+                        data={'hwversion': c.hwversion, 'mac': c.MAC, 'version': c.version}
+                        devices[host]=data
+        return devices
+
+    def upgradeOnlineDevices(self):
+        devices=self.retrieveOnlineDevices()
+        if devices:
+            data=self.retrieveFirmware(True)
+            if data:
+                try:
+                    version=data.get('version')
+                    if version:
+                        for host in devices.keys():
+                            v=devices[host]['version']
+                            if v<version:
+                                self.logger.info('SIO device %s has version %s. Upgrading to version %s...' % (host, v, version))
+                                self.upgrade(host)
+                except:
+                    pass
+
+    def upgradeAll(self):
         if not self._hosts:
             self.scan()
         if self._hosts:
