@@ -693,10 +693,6 @@ class MBIOGatewayMetzConnect(MBIOGateway):
         for device in self.devices:
             device.auto()
 
-    def manual(self):
-        for device in self.devices:
-            device.manual()
-
 
 class MBIOGateways(Items):
     def __init__(self, logger):
