@@ -414,7 +414,7 @@ class MBIOLink(object):
             io='%umbio%u' % (station, item)
             self.logger.warning('<IOMAPPING %s<->%s' % (key, io))
             value.config.set('iomaptag', io)
-            value.config.set('iomapinstallation', station)
+            value.config.set('iomapinstallation', installation)
             value.config.set('iomapcpu', station)
             value.config.set('iomapitem', item)
             value.config.set('iomapkey', 'r_%u_%u_mbio_%d_0' % (installation, station, item))
@@ -433,14 +433,18 @@ class MBIOLink(object):
             try:
                 data=description.split('|')
                 label=data[0]
+                low=data[1]
+                normal=data[2]
+                high=data[3]
 
                 if label.lower()==value.key:
                     return
 
-                # TODO: We may here extract low/normal/high states
-
                 self.logger.warning('<IOUPDATEDESCRIPTION %s [%s]' % (key, label))
                 value.config.set('iodescription', label.strip())
+                value.config.set('iolow', low.strip())
+                value.config.set('ionormal', normal.strip())
+                value.config.set('iohigh', high.strip())
             except:
                 pass
 
