@@ -485,6 +485,12 @@ class MBIOTask(object):
 
         return False
 
+    def auto(self):
+        self.values.auto()
+
+    def isManualValue(self):
+        return self.values.isManual()
+
 
 class MBIOTasks(Items):
     def __init__(self, logger):
