@@ -325,7 +325,14 @@ class MBIOValue(object):
             return None
         if self.isDigital():
             if self.value:
+                label=self.config.iohigh
+                if label:
+                    return label
                 return 'ON'
+
+            label=self.config.iolow
+            if label:
+                return label
             return 'OFF'
         try:
             decimal=float(self.resolution) % 1
@@ -661,6 +668,13 @@ class MBIOValueDigital(MBIOValue):
                 return value
             if type(value) is int:
                 return bool(value)
+            if type(value) is str:
+                value=value.lower()
+                if value in ['false', 'no', 'off']:
+                    return False
+                if value in ['true', 'yes', 'on']:
+                    return True
+                return bool(int(value))
             if value is None:
                 return False
             return bool(value)
@@ -956,6 +970,16 @@ class MBIOValues(Items):
         except:
             pass
 
+    def isManual(self):
+        for value in self:
+            if value.isManual():
+                return True
+        return False
+
+    def auto(self):
+        for value in self:
+            value.auto()
+
     # Force return None if called with an unknown attribute (value name)
     def __getattr__(self, name):
         try:
